@@ -7,6 +7,10 @@ class MarginMode(str, Enum):
     cross margin/futures margin_mode and
     isolated margin/futures margin_mode
     """
+
     CROSS = "cross"
     ISOLATED = "isolated"
-    NONE = ''
+    NONE = ""
+
+    def __str__(self):
+        return f"{self.value.lower()}"
