@@ -1,8 +1,9 @@
-""" Bitmart exchange subclass """
+"""Bitmart exchange subclass"""
+
 import logging
-from typing import Dict
 
 from freqtrade.exchange import Exchange
+from freqtrade.exchange.exchange_types import FtHas
 
 
 logger = logging.getLogger(__name__)
@@ -14,7 +15,8 @@ class Bitmart(Exchange):
     with this exchange.
     """
 
-    _ft_has: Dict = {
+    _ft_has: FtHas = {
         "stoploss_on_exchange": False,  # Bitmart API does not support stoploss orders
         "ohlcv_candle_limit": 200,
+        "trades_has_history": False,  # Endpoint doesn't seem to support pagination
     }
