@@ -1,8 +1,9 @@
-"""Kucoin exchange subclass."""
+"""Bitvavo exchange subclass."""
+
 import logging
-from typing import Dict
 
 from freqtrade.exchange import Exchange
+from freqtrade.exchange.exchange_types import FtHas
 
 
 logger = logging.getLogger(__name__)
@@ -18,6 +19,6 @@ class Bitvavo(Exchange):
     may still not work as expected.
     """
 
-    _ft_has: Dict = {
+    _ft_has: FtHas = {
         "ohlcv_candle_limit": 1440,
     }
