@@ -1,7 +1,6 @@
 import logging
-import sys
 from pathlib import Path
-from typing import Any, Dict
+from typing import Any
 
 from catboost import CatBoostClassifier, Pool
 
@@ -22,7 +21,7 @@ class CatboostClassifier(BaseClassifierModel):
     top level config.json file.
     """
 
-    def fit(self, data_dictionary: Dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
+    def fit(self, data_dictionary: dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
         """
         User sets up the training and test data to fit their desired model here
         :param data_dictionary: the dictionary holding all data for train, test,
@@ -46,14 +45,17 @@ class CatboostClassifier(BaseClassifierModel):
 
         cbr = CatBoostClassifier(
             allow_writing_files=True,
-            loss_function='MultiClass',
+            loss_function="MultiClass",
             train_dir=Path(dk.data_path),
             **self.model_training_parameters,
         )
 
         init_model = self.get_init_model(dk.pair)
 
-        cbr.fit(X=train_data, eval_set=test_data, init_model=init_model,
-                log_cout=sys.stdout, log_cerr=sys.stderr)
+        cbr.fit(
+            X=train_data,
+            eval_set=test_data,
+            init_model=init_model,
+        )
 
         return cbr
