@@ -1,7 +1,6 @@
 import logging
-import sys
 from pathlib import Path
-from typing import Any, Dict
+from typing import Any
 
 from catboost import CatBoostClassifier, Pool
 
@@ -23,7 +22,7 @@ class CatboostClassifierMultiTarget(BaseClassifierModel):
     top level config.json file.
     """
 
-    def fit(self, data_dictionary: Dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
+    def fit(self, data_dictionary: dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
         """
         User sets up the training and test data to fit their desired model here
         :param data_dictionary: the dictionary holding all data for train, test,
@@ -33,7 +32,7 @@ class CatboostClassifierMultiTarget(BaseClassifierModel):
 
         cbc = CatBoostClassifier(
             allow_writing_files=True,
-            loss_function='MultiClass',
+            loss_function="MultiClass",
             train_dir=Path(dk.data_path),
             **self.model_training_parameters,
         )
@@ -45,10 +44,10 @@ class CatboostClassifierMultiTarget(BaseClassifierModel):
 
         eval_sets = [None] * y.shape[1]
 
-        if self.freqai_info.get('data_split_parameters', {}).get('test_size', 0.1) != 0:
-            eval_sets = [None] * data_dictionary['test_labels'].shape[1]
+        if self.freqai_info.get("data_split_parameters", {}).get("test_size", 0.1) != 0:
+            eval_sets = [None] * data_dictionary["test_labels"].shape[1]
 
-            for i in range(data_dictionary['test_labels'].shape[1]):
+            for i in range(data_dictionary["test_labels"].shape[1]):
                 eval_sets[i] = Pool(
                     data=data_dictionary["test_features"],
                     label=data_dictionary["test_labels"].iloc[:, i],
@@ -64,13 +63,15 @@ class CatboostClassifierMultiTarget(BaseClassifierModel):
 
         fit_params = []
         for i in range(len(eval_sets)):
-            fit_params.append({
-                'eval_set': eval_sets[i], 'init_model': init_models[i],
-                'log_cout': sys.stdout, 'log_cerr': sys.stderr,
-            })
+            fit_params.append(
+                {
+                    "eval_set": eval_sets[i],
+                    "init_model": init_models[i],
+                }
+            )
 
         model = FreqaiMultiOutputClassifier(estimator=cbc)
-        thread_training = self.freqai_info.get('multitarget_parallel_training', False)
+        thread_training = self.freqai_info.get("multitarget_parallel_training", False)
         if thread_training:
             model.n_jobs = y.shape[1]
         model.fit(X=X, y=y, sample_weight=sample_weight, fit_params=fit_params)
