@@ -1,5 +1,5 @@
 import logging
-from typing import Any, Dict
+from typing import Any
 
 from xgboost import XGBRegressor
 
@@ -21,7 +21,7 @@ class XGBoostRegressor(BaseRegressionModel):
     top level config.json file.
     """
 
-    def fit(self, data_dictionary: Dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
+    def fit(self, data_dictionary: dict, dk: FreqaiDataKitchen, **kwargs) -> Any:
         """
         User sets up the training and test data to fit their desired model here
         :param data_dictionary: the dictionary holding all data for train, test,
@@ -36,15 +36,8 @@ class XGBoostRegressor(BaseRegressionModel):
             eval_set = None
             eval_weights = None
         else:
-            eval_set = [
-                (data_dictionary["test_features"],
-                 data_dictionary["test_labels"]),
-                (X, y)
-            ]
-            eval_weights = [
-                data_dictionary['test_weights'],
-                data_dictionary['train_weights']
-            ]
+            eval_set = [(data_dictionary["test_features"], data_dictionary["test_labels"]), (X, y)]
+            eval_weights = [data_dictionary["test_weights"], data_dictionary["train_weights"]]
 
         sample_weight = data_dictionary["train_weights"]
 
@@ -53,8 +46,14 @@ class XGBoostRegressor(BaseRegressionModel):
         model = XGBRegressor(**self.model_training_parameters)
 
         model.set_params(callbacks=[TBCallback(dk.data_path)])
-        model.fit(X=X, y=y, sample_weight=sample_weight, eval_set=eval_set,
-                  sample_weight_eval_set=eval_weights, xgb_model=xgb_model)
+        model.fit(
+            X=X,
+            y=y,
+            sample_weight=sample_weight,
+            eval_set=eval_set,
+            sample_weight_eval_set=eval_weights,
+            xgb_model=xgb_model,
+        )
         # set the callbacks to empty so that we can serialize to disk later
         model.set_params(callbacks=[])
 
