@@ -5,8 +5,10 @@ from typing import Any
 from torch.utils.tensorboard import SummaryWriter
 from xgboost import callback
 
-from freqtrade.freqai.tensorboard.base_tensorboard import (BaseTensorBoardCallback,
-                                                           BaseTensorboardLogger)
+from freqtrade.freqai.tensorboard.base_tensorboard import (
+    BaseTensorBoardCallback,
+    BaseTensorboardLogger,
+)
 
 
 logger = logging.getLogger(__name__)
@@ -29,7 +31,6 @@ class TensorboardLogger(BaseTensorboardLogger):
 
 
 class TensorBoardCallback(BaseTensorBoardCallback):
-
     def __init__(self, logdir: Path, activate: bool = True):
         self.activate = activate
         if self.activate:
@@ -44,10 +45,10 @@ class TensorBoardCallback(BaseTensorBoardCallback):
             return False
 
         evals = ["validation", "train"]
-        for metric, eval in zip(evals_log.items(), evals):
+        for metric, eval_ in zip(evals_log.items(), evals, strict=False):
             for metric_name, log in metric[1].items():
                 score = log[-1][0] if isinstance(log[-1], tuple) else log[-1]
-                self.writer.add_scalar(f"{eval}-{metric_name}", score, epoch)
+                self.writer.add_scalar(f"{eval_}-{metric_name}", score, epoch)
 
         return False
 
