@@ -5,5 +5,8 @@ from freqtrade.persistence.key_value_store import KeyStoreKeys, KeyValueStore
 from freqtrade.persistence.models import init_db
 from freqtrade.persistence.pairlock_middleware import PairLocks
 from freqtrade.persistence.trade_model import LocalTrade, Order, Trade
-from freqtrade.persistence.usedb_context import (FtNoDBContext, disable_database_use,
-                                                 enable_database_use)
+from freqtrade.persistence.usedb_context import (
+    FtNoDBContext,
+    disable_database_use,
+    enable_database_use,
+)
