@@ -1,7 +1,5 @@
-
 import logging
 from datetime import datetime, timedelta
-from typing import Optional
 
 from freqtrade.constants import LongShort
 from freqtrade.persistence import Trade
@@ -12,7 +10,6 @@ logger = logging.getLogger(__name__)
 
 
 class CooldownPeriod(IProtection):
-
     has_global_stop: bool = False
     has_local_stop: bool = True
 
@@ -20,19 +17,19 @@ class CooldownPeriod(IProtection):
         """
         LockReason to use
         """
-        return (f'Cooldown period for {self.stop_duration_str}.')
+        return f"Cooldown period for {self.unlock_reason_time_element}."
 
     def short_desc(self) -> str:
         """
-        Short method description - used for startup-messages
+        Short method description - used for startup messages
         """
-        return (f"{self.name} - Cooldown period of {self.stop_duration_str}.")
+        return f"{self.name} - Cooldown period {self.unlock_reason_time_element}."
 
-    def _cooldown_period(self, pair: str, date_now: datetime) -> Optional[ProtectionReturn]:
+    def _cooldown_period(self, pair: str, date_now: datetime) -> ProtectionReturn | None:
         """
         Get last trade for this pair
         """
-        look_back_until = date_now - timedelta(minutes=self._stop_duration)
+        look_back_until = date_now - timedelta(minutes=self._lookback_period)
         # filters = [
         #     Trade.is_open.is_(False),
         #     Trade.close_date > look_back_until,
@@ -44,8 +41,8 @@ class CooldownPeriod(IProtection):
             # Get latest trade
             # Ignore type error as we know we only get closed trades.
             trade = sorted(trades, key=lambda t: t.close_date)[-1]  # type: ignore
-            self.log_once(f"Cooldown for {pair} for {self.stop_duration_str}.", logger.info)
-            until = self.calculate_lock_end([trade], self._stop_duration)
+            self.log_once(f"Cooldown for {pair} {self.unlock_reason_time_element}.", logger.info)
+            until = self.calculate_lock_end([trade])
 
             return ProtectionReturn(
                 lock=True,
@@ -55,7 +52,7 @@ class CooldownPeriod(IProtection):
 
         return None
 
-    def global_stop(self, date_now: datetime, side: LongShort) -> Optional[ProtectionReturn]:
+    def global_stop(self, date_now: datetime, side: LongShort) -> ProtectionReturn | None:
         """
         Stops trading (position entering) for all pairs
         This must evaluate to true for the whole period of the "cooldown period".
@@ -66,7 +63,8 @@ class CooldownPeriod(IProtection):
         return None
 
     def stop_per_pair(
-            self, pair: str, date_now: datetime, side: LongShort) -> Optional[ProtectionReturn]:
+        self, pair: str, date_now: datetime, side: LongShort
+    ) -> ProtectionReturn | None:
         """
         Stops trading (position entering) for this pair
         This must evaluate to true for the whole period of the "cooldown period".
