@@ -1,4 +1,5 @@
-from typing import Any, AsyncIterator, Dict, Optional
+from collections.abc import AsyncIterator
+from typing import Any
 from uuid import uuid4
 
 from fastapi import Depends, HTTPException
@@ -13,14 +14,13 @@ from freqtrade.rpc.rpc import RPC, RPCException
 from .webserver import ApiServer
 
 
-def get_rpc_optional() -> Optional[RPC]:
+def get_rpc_optional() -> RPC | None:
     if ApiServer._has_rpc:
         return ApiServer._rpc
     return None
 
 
-async def get_rpc() -> Optional[AsyncIterator[RPC]]:
-
+async def get_rpc() -> AsyncIterator[RPC] | None:
     _rpc = get_rpc_optional()
     if _rpc:
         request_id = str(uuid4())
@@ -33,15 +33,15 @@ async def get_rpc() -> Optional[AsyncIterator[RPC]]:
             _request_id_ctx_var.reset(ctx_token)
 
     else:
-        raise RPCException('Bot is not in the correct state')
+        raise RPCException("Bot is not in the correct state")
 
 
-def get_config() -> Dict[str, Any]:
+def get_config() -> dict[str, Any]:
     return ApiServer._config
 
 
-def get_api_config() -> Dict[str, Any]:
-    return ApiServer._config['api_server']
+def get_api_config() -> dict[str, Any]:
+    return ApiServer._config["api_server"]
 
 
 def _generate_exchange_key(config: Config) -> str:
@@ -55,8 +55,8 @@ def get_exchange(config=Depends(get_config)):
     exchange_key = _generate_exchange_key(config)
     if not (exchange := ApiBG.exchanges.get(exchange_key)):
         from freqtrade.resolvers import ExchangeResolver
-        exchange = ExchangeResolver.load_exchange(
-            config, validate=False, load_leverage_tiers=False)
+
+        exchange = ExchangeResolver.load_exchange(config, validate=False, load_leverage_tiers=False)
         ApiBG.exchanges[exchange_key] = exchange
     return exchange
 
@@ -66,7 +66,6 @@ def get_message_stream():
 
 
 def is_webserver_mode(config=Depends(get_config)):
-    if config['runmode'] != RunMode.WEBSERVER:
-        raise HTTPException(status_code=503,
-                            detail='Bot is not in the correct state.')
+    if config["runmode"] != RunMode.WEBSERVER:
+        raise HTTPException(status_code=503, detail="Bot is not in the correct state.")
     return None
