@@ -1,5 +1,5 @@
 # isort: off
-from freqtrade.rpc.api_server.ws.types import WebSocketType  # noqa: F401
+from freqtrade.rpc.api_server.ws.ws_types import WebSocketType  # noqa: F401
 from freqtrade.rpc.api_server.ws.proxy import WebSocketProxy  # noqa: F401
 from freqtrade.rpc.api_server.ws.serializer import HybridJSONWebSocketSerializer  # noqa: F401
 from freqtrade.rpc.api_server.ws.channel import WebSocketChannel  # noqa: F401
