@@ -1,5 +1,5 @@
 from datetime import datetime
-from typing import Any, List, Literal, Optional, TypedDict, Union
+from typing import Any, Literal, TypedDict
 
 from freqtrade.constants import PairWithTimeframe
 from freqtrade.enums import RPCMessageType
@@ -15,12 +15,14 @@ class RPCSendMsgBase(TypedDict):
 
 class RPCStatusMsg(RPCSendMsgBase):
     """Used for Status, Startup and Warning messages"""
+
     type: Literal[RPCMessageType.STATUS, RPCMessageType.STARTUP, RPCMessageType.WARNING]
     status: str
 
 
 class RPCStrategyMsg(RPCSendMsgBase):
     """Used for Status, Startup and Warning messages"""
+
     type: Literal[RPCMessageType.STRATEGY_MSG]
     msg: str
 
@@ -29,7 +31,7 @@ class RPCProtectionMsg(RPCSendMsgBase):
     type: Literal[RPCMessageType.PROTECTION_TRIGGER, RPCMessageType.PROTECTION_TRIGGER_GLOBAL]
     id: int
     pair: str
-    base_currency: Optional[str]
+    base_currency: str | None
     lock_time: str
     lock_timestamp: int
     lock_end_time: str
@@ -41,28 +43,29 @@ class RPCProtectionMsg(RPCSendMsgBase):
 
 class RPCWhitelistMsg(RPCSendMsgBase):
     type: Literal[RPCMessageType.WHITELIST]
-    data: List[str]
+    data: list[str]
 
 
 class __RPCEntryExitMsgBase(RPCSendMsgBase):
     trade_id: int
-    buy_tag: Optional[str]
-    enter_tag: Optional[str]
+    buy_tag: str | None
+    enter_tag: str | None
     exchange: str
     pair: str
     base_currency: str
     quote_currency: str
-    leverage: Optional[float]
+    leverage: float | None
     direction: str
-    limit: float
+    limit: float  # Deprecated, use order_rate instead
+    order_rate: float
     open_rate: float
     order_type: str
     stake_amount: float
     stake_currency: str
-    fiat_currency: Optional[str]
+    fiat_currency: str | None
     amount: float
     open_date: datetime
-    current_rate: Optional[float]
+    current_rate: float | None
     sub_trade: bool
 
 
@@ -82,11 +85,10 @@ class RPCExitMsg(__RPCEntryExitMsgBase):
     close_rate: float
     profit_amount: float
     profit_ratio: float
-    exit_reason: Optional[str]
+    exit_reason: str | None
     close_date: datetime
-    # current_rate: Optional[float]
-    order_rate: Optional[float]
-    final_profit_ratio: Optional[float]
+    # current_rate: float | None
+    final_profit_ratio: float | None
     is_final_exit: bool
 
 
@@ -96,7 +98,7 @@ class RPCExitCancelMsg(__RPCEntryExitMsgBase):
     gain: ProfitLossStr
     profit_amount: float
     profit_ratio: float
-    exit_reason: Optional[str]
+    exit_reason: str | None
     close_date: datetime
 
 
@@ -108,28 +110,30 @@ class _AnalyzedDFData(TypedDict):
 
 class RPCAnalyzedDFMsg(RPCSendMsgBase):
     """New Analyzed dataframe message"""
+
     type: Literal[RPCMessageType.ANALYZED_DF]
     data: _AnalyzedDFData
 
 
 class RPCNewCandleMsg(RPCSendMsgBase):
     """New candle ping message, issued once per new candle/pair"""
+
     type: Literal[RPCMessageType.NEW_CANDLE]
     data: PairWithTimeframe
 
 
-RPCOrderMsg = Union[RPCEntryMsg, RPCExitMsg, RPCExitCancelMsg, RPCCancelMsg]
+RPCOrderMsg = RPCEntryMsg | RPCExitMsg | RPCExitCancelMsg | RPCCancelMsg
 
 
-RPCSendMsg = Union[
-    RPCStatusMsg,
-    RPCStrategyMsg,
-    RPCProtectionMsg,
-    RPCWhitelistMsg,
-    RPCEntryMsg,
-    RPCCancelMsg,
-    RPCExitMsg,
-    RPCExitCancelMsg,
-    RPCAnalyzedDFMsg,
-    RPCNewCandleMsg
-    ]
+RPCSendMsg = (
+    RPCStatusMsg
+    | RPCStrategyMsg
+    | RPCProtectionMsg
+    | RPCWhitelistMsg
+    | RPCEntryMsg
+    | RPCCancelMsg
+    | RPCExitMsg
+    | RPCExitCancelMsg
+    | RPCAnalyzedDFMsg
+    | RPCNewCandleMsg
+)
