@@ -1,34 +1,41 @@
 import re
-from datetime import datetime, timezone
-from typing import Optional
+from datetime import UTC, datetime
+from time import time
 
-import arrow
+import humanize
 
 from freqtrade.constants import DATETIME_PRINT_FORMAT
 
 
 def dt_now() -> datetime:
     """Return the current datetime in UTC."""
-    return datetime.now(timezone.utc)
+    return datetime.now(UTC)
 
 
-def dt_utc(year: int, month: int, day: int, hour: int = 0, minute: int = 0, second: int = 0,
-           microsecond: int = 0) -> datetime:
+def dt_utc(
+    year: int,
+    month: int,
+    day: int,
+    hour: int = 0,
+    minute: int = 0,
+    second: int = 0,
+    microsecond: int = 0,
+) -> datetime:
     """Return a datetime in UTC."""
-    return datetime(year, month, day, hour, minute, second, microsecond, tzinfo=timezone.utc)
+    return datetime(year, month, day, hour, minute, second, microsecond, tzinfo=UTC)
 
 
-def dt_ts(dt: Optional[datetime] = None) -> int:
+def dt_ts(dt: datetime | None = None) -> int:
     """
     Return dt in ms as a timestamp in UTC.
     If dt is None, return the current datetime in UTC.
     """
     if dt:
         return int(dt.timestamp() * 1000)
-    return int(dt_now().timestamp() * 1000)
+    return int(time() * 1000)
 
 
-def dt_ts_def(dt: Optional[datetime], default: int = 0) -> int:
+def dt_ts_def(dt: datetime | None, default: int = 0) -> int:
     """
     Return dt in ms as a timestamp in UTC.
     If dt is None, return the given default.
@@ -38,7 +45,7 @@ def dt_ts_def(dt: Optional[datetime], default: int = 0) -> int:
     return default
 
 
-def dt_ts_none(dt: Optional[datetime]) -> Optional[int]:
+def dt_ts_none(dt: datetime | None) -> int | None:
     """
     Return dt in ms as a timestamp in UTC.
     If dt is None, return the given default.
@@ -61,31 +68,29 @@ def dt_from_ts(timestamp: float) -> datetime:
     if timestamp > 1e10:
         # Timezone in ms - convert to seconds
         timestamp /= 1000
-    return datetime.fromtimestamp(timestamp, tz=timezone.utc)
+    return datetime.fromtimestamp(timestamp, tz=UTC)
 
 
 def shorten_date(_date: str) -> str:
     """
     Trim the date so it fits on small screens
     """
-    new_date = re.sub('seconds?', 'sec', _date)
-    new_date = re.sub('minutes?', 'min', new_date)
-    new_date = re.sub('hours?', 'h', new_date)
-    new_date = re.sub('days?', 'd', new_date)
-    new_date = re.sub('^an?', '1', new_date)
+    new_date = re.sub("seconds?", "sec", _date)
+    new_date = re.sub("minutes?", "min", new_date)
+    new_date = re.sub("hours?", "h", new_date)
+    new_date = re.sub("days?", "d", new_date)
+    new_date = re.sub("^an?", "1", new_date)
     return new_date
 
 
-def dt_humanize(dt: datetime, **kwargs) -> str:
+def dt_humanize_delta(dt: datetime):
     """
-    Return a humanized string for the given datetime.
-    :param dt: datetime to humanize
-    :param kwargs: kwargs to pass to arrow's humanize()
+    Return a humanized string for the given timedelta.
     """
-    return arrow.get(dt).humanize(**kwargs)
+    return humanize.naturaltime(dt)
 
 
-def format_date(date: Optional[datetime]) -> str:
+def format_date(date: datetime | None) -> str:
     """
     Return a formatted date string.
     Returns an empty string if date is None.
@@ -93,12 +98,21 @@ def format_date(date: Optional[datetime]) -> str:
     """
     if date:
         return date.strftime(DATETIME_PRINT_FORMAT)
-    return ''
+    return ""
 
 
-def format_ms_time(date: int) -> str:
+def format_ms_time(date: int | float) -> str:
     """
     convert MS date to readable format.
     : epoch-string in ms
     """
-    return datetime.fromtimestamp(date / 1000.0).strftime('%Y-%m-%dT%H:%M:%S')
+    return dt_from_ts(date).strftime("%Y-%m-%dT%H:%M:%S")
+
+
+def format_ms_time_det(date: int | float) -> str:
+    """
+    convert MS date to readable format - detailed.
+    : epoch-string in ms
+    """
+    # return dt_from_ts(date).isoformat(timespec="milliseconds")
+    return dt_from_ts(date).strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3]
