@@ -1,10 +1,9 @@
 # pragma pylint: disable=W0603
-""" Wallet """
+"""Wallet"""
 
 import logging
-from copy import deepcopy
 from datetime import datetime, timedelta
-from typing import Dict, NamedTuple, Optional
+from typing import Literal, NamedTuple
 
 from freqtrade.constants import UNLIMITED_STAKE_AMOUNT, Config, IntOrInf
 from freqtrade.enums import RunMode, TradingMode
@@ -29,21 +28,28 @@ class Wallet(NamedTuple):
 class PositionWallet(NamedTuple):
     symbol: str
     position: float = 0
-    leverage: float = 0
+    leverage: float | None = 0  # Don't use this - it's not guaranteed to be set
     collateral: float = 0
-    side: str = 'long'
+    side: str = "long"
 
 
 class Wallets:
-
     def __init__(self, config: Config, exchange: Exchange, is_backtest: bool = False) -> None:
         self._config = config
         self._is_backtest = is_backtest
         self._exchange = exchange
-        self._wallets: Dict[str, Wallet] = {}
-        self._positions: Dict[str, PositionWallet] = {}
-        self.start_cap = config['dry_run_wallet']
-        self._last_wallet_refresh: Optional[datetime] = None
+        self._wallets: dict[str, Wallet] = {}
+        self._positions: dict[str, PositionWallet] = {}
+        self._start_cap: dict[str, float] = {}
+
+        self._stake_currency = self._exchange.get_proxy_coin()
+
+        if isinstance(_start_cap := config["dry_run_wallet"], float | int):
+            self._start_cap[self._stake_currency] = _start_cap
+        else:
+            self._start_cap = _start_cap
+
+        self._last_wallet_refresh: datetime | None = None
         self.update()
 
     def get_free(self, currency: str) -> float:
@@ -67,10 +73,33 @@ class Wallets:
         else:
             return 0
 
+    def get_collateral(self) -> float:
+        """
+        Get total collateral for liquidation price calculation.
+        """
+        if self._config.get("margin_mode") == "cross":
+            # free includes all balances and, combined with position collateral,
+            # is used as "wallet balance".
+            return self.get_free(self._stake_currency) + sum(
+                pos.collateral for pos in self._positions.values()
+            )
+        return self.get_total(self._stake_currency)
+
+    def get_owned(self, pair: str, base_currency: str) -> float:
+        """
+        Get currently owned value.
+        Designed to work across both spot and futures.
+        """
+        if self._config.get("trading_mode", "spot") != TradingMode.FUTURES:
+            return self.get_total(base_currency) or 0
+        if pos := self._positions.get(pair):
+            return pos.position
+        return 0
+
     def _update_dry(self) -> None:
         """
         Update from database in dry-run mode
-        - Apply apply profits of closed trades on top of stake amount
+        - Apply profits of closed trades on top of stake amount
         - Subtract currently tied up stake_amount in open trades
         - update balances for currencies currently in trades
         """
@@ -83,81 +112,102 @@ class Wallets:
             tot_profit = Trade.get_total_closed_profit()
         else:
             # Backtest mode
-            tot_profit = LocalTrade.total_profit
+            tot_profit = LocalTrade.bt_total_profit
         tot_profit += sum(trade.realized_profit for trade in open_trades)
         tot_in_trades = sum(trade.stake_amount for trade in open_trades)
         used_stake = 0.0
 
-        if self._config.get('trading_mode', 'spot') != TradingMode.FUTURES:
-            current_stake = self.start_cap + tot_profit - tot_in_trades
-            total_stake = current_stake
+        if self._config.get("trading_mode", "spot") != TradingMode.FUTURES:
             for trade in open_trades:
                 curr = self._exchange.get_pair_base_currency(trade.pair)
+                used_stake += sum(
+                    o.stake_amount for o in trade.open_orders if o.ft_order_side == trade.entry_side
+                )
+                pending = sum(
+                    o.amount
+                    for o in trade.open_orders
+                    if o.amount and o.ft_order_side == trade.exit_side
+                )
+                curr_wallet_bal = self._start_cap.get(curr, 0)
+
                 _wallets[curr] = Wallet(
                     curr,
-                    trade.amount,
-                    0,
-                    trade.amount
+                    curr_wallet_bal + trade.amount - pending,
+                    pending,
+                    trade.amount + curr_wallet_bal,
                 )
         else:
-            tot_in_trades = 0
             for position in open_trades:
-                # size = self._exchange._contracts_to_amount(position.pair, position['contracts'])
-                size = position.amount
-                collateral = position.stake_amount
-                leverage = position.leverage
-                tot_in_trades += collateral
                 _positions[position.pair] = PositionWallet(
-                    position.pair, position=size,
-                    leverage=leverage,
-                    collateral=collateral,
-                    side=position.trade_direction
+                    position.pair,
+                    position=position.amount,
+                    leverage=position.leverage,
+                    collateral=position.stake_amount,
+                    side=position.trade_direction,
                 )
-            current_stake = self.start_cap + tot_profit - tot_in_trades
-            used_stake = tot_in_trades
-            total_stake = current_stake + tot_in_trades
 
-        _wallets[self._config['stake_currency']] = Wallet(
-            currency=self._config['stake_currency'],
-            free=current_stake,
+            used_stake = tot_in_trades
+
+        cross_margin = 0.0
+        if self._config.get("margin_mode") == "cross":
+            # In cross-margin mode, the total balance is used as collateral.
+            # This is moved as "free" into the stake currency balance.
+            # strongly tied to the get_collateral() implementation.
+            for curr, bal in self._start_cap.items():
+                if curr == self._stake_currency:
+                    continue
+                rate = self._exchange.get_conversion_rate(curr, self._stake_currency)
+                if rate:
+                    cross_margin += bal * rate
+
+        current_stake = self._start_cap.get(self._stake_currency, 0) + tot_profit - tot_in_trades
+        total_stake = current_stake + used_stake
+
+        _wallets[self._stake_currency] = Wallet(
+            currency=self._stake_currency,
+            free=current_stake + cross_margin,
             used=used_stake,
-            total=total_stake
+            total=total_stake,
         )
+        for currency, bal in self._start_cap.items():
+            if currency not in _wallets:
+                _wallets[currency] = Wallet(currency, bal, 0, bal)
+
         self._wallets = _wallets
         self._positions = _positions
 
     def _update_live(self) -> None:
         balances = self._exchange.get_balances()
+        _wallets = {}
 
         for currency in balances:
             if isinstance(balances[currency], dict):
-                self._wallets[currency] = Wallet(
+                _wallets[currency] = Wallet(
                     currency,
-                    balances[currency].get('free'),
-                    balances[currency].get('used'),
-                    balances[currency].get('total')
+                    balances[currency].get("free", 0),
+                    balances[currency].get("used", 0),
+                    balances[currency].get("total", 0),
                 )
-        # Remove currencies no longer in get_balances output
-        for currency in deepcopy(self._wallets):
-            if currency not in balances:
-                del self._wallets[currency]
 
         positions = self._exchange.fetch_positions()
-        self._positions = {}
+        _parsed_positions = {}
         for position in positions:
-            symbol = position['symbol']
-            if position['side'] is None or position['collateral'] == 0.0:
+            symbol = position["symbol"]
+            if position["side"] is None or position["collateral"] == 0.0:
                 # Position is not open ...
                 continue
-            size = self._exchange._contracts_to_amount(symbol, position['contracts'])
-            collateral = safe_value_fallback(position, 'collateral', 'initialMargin', 0.0)
-            leverage = position['leverage']
-            self._positions[symbol] = PositionWallet(
-                symbol, position=size,
+            size = self._exchange._contracts_to_amount(symbol, position["contracts"])
+            collateral = safe_value_fallback(position, "initialMargin", "collateral", 0.0)
+            leverage = position.get("leverage")
+            _parsed_positions[symbol] = PositionWallet(
+                symbol,
+                position=size,
                 leverage=leverage,
                 collateral=collateral,
-                side=position['side']
+                side=position["side"],
             )
+        self._positions = _parsed_positions
+        self._wallets = _wallets
 
     def update(self, require_update: bool = True) -> None:
         """
@@ -173,18 +223,17 @@ class Wallets:
             or self._last_wallet_refresh is None
             or (self._last_wallet_refresh + timedelta(seconds=3600) < now)
         ):
-            if (not self._config['dry_run'] or self._config.get('runmode') == RunMode.LIVE):
+            if not self._config["dry_run"] or self._config.get("runmode") == RunMode.LIVE:
                 self._update_live()
             else:
                 self._update_dry()
-            if not self._is_backtest:
-                logger.info('Wallets synced.')
+            self._local_log("Wallets synced.")
             self._last_wallet_refresh = dt_now()
 
-    def get_all_balances(self) -> Dict[str, Wallet]:
+    def get_all_balances(self) -> dict[str, Wallet]:
         return self._wallets
 
-    def get_all_positions(self) -> Dict[str, PositionWallet]:
+    def get_all_positions(self) -> dict[str, PositionWallet]:
         return self._positions
 
     def _check_exit_amount(self, trade: Trade) -> bool:
@@ -222,12 +271,14 @@ class Wallets:
         or by using current balance subtracting
         """
         if "available_capital" in self._config:
-            return self._config['available_capital']
+            return self._config["available_capital"]
         else:
             tot_profit = Trade.get_total_closed_profit()
             open_stakes = Trade.total_open_trades_stakes()
-            available_balance = self.get_free(self._config['stake_currency'])
-            return available_balance - tot_profit + open_stakes
+            available_balance = self.get_free(self._stake_currency)
+            return (available_balance - tot_profit + open_stakes) * self._config[
+                "tradable_balance_ratio"
+            ]
 
     def get_total_stake_amount(self):
         """
@@ -238,7 +289,7 @@ class Wallets:
         """
         val_tied_up = Trade.total_open_trades_stakes()
         if "available_capital" in self._config:
-            starting_balance = self._config['available_capital']
+            starting_balance = self._config["available_capital"]
             tot_profit = Trade.get_total_closed_profit()
             available_amount = starting_balance + tot_profit
 
@@ -246,8 +297,9 @@ class Wallets:
             # Ensure <tradable_balance_ratio>% is used from the overall balance
             # Otherwise we'd risk lowering stakes with each open trade.
             # (tied up + current free) * ratio) - tied up
-            available_amount = ((val_tied_up + self.get_free(self._config['stake_currency'])) *
-                                self._config['tradable_balance_ratio'])
+            available_amount = (val_tied_up + self.get_free(self._stake_currency)) * self._config[
+                "tradable_balance_ratio"
+            ]
         return available_amount
 
     def get_available_stake_amount(self) -> float:
@@ -258,11 +310,12 @@ class Wallets:
         (<open_trade stakes> + free amount) * tradable_balance_ratio - <open_trade stakes>
         """
 
-        free = self.get_free(self._config['stake_currency'])
+        free = self.get_free(self._stake_currency)
         return min(self.get_total_stake_amount() - Trade.total_open_trades_stakes(), free)
 
-    def _calculate_unlimited_stake_amount(self, available_amount: float,
-                                          val_tied_up: float, max_open_trades: IntOrInf) -> float:
+    def _calculate_unlimited_stake_amount(
+        self, available_amount: float, val_tied_up: float, max_open_trades: IntOrInf
+    ) -> float:
         """
         Calculate stake amount for "unlimited" stake amount
         :return: 0 if max number of trades reached, else stake_amount to use.
@@ -282,10 +335,10 @@ class Wallets:
         :raise: DependencyException if balance is lower than stake-amount
         """
 
-        if self._config['amend_last_stake_amount']:
+        if self._config["amend_last_stake_amount"]:
             # Remaining amount needs to be at least stake_amount * last_stake_amount_min_ratio
             # Otherwise the remaining amount is too low to trade.
-            if available_amount > (stake_amount * self._config['last_stake_amount_min_ratio']):
+            if available_amount > (stake_amount * self._config["last_stake_amount_min_ratio"]):
                 stake_amount = min(stake_amount, available_amount)
             else:
                 stake_amount = 0
@@ -296,42 +349,44 @@ class Wallets:
                 f"lower than stake amount ({stake_amount} {self._config['stake_currency']})"
             )
 
-        return stake_amount
+        return max(stake_amount, 0)
 
     def get_trade_stake_amount(
-            self, pair: str, max_open_trades: IntOrInf, edge=None, update: bool = True) -> float:
+        self, pair: str, max_open_trades: IntOrInf, update: bool = True
+    ) -> float:
         """
         Calculate stake amount for the trade
         :return: float: Stake amount
         :raise: DependencyException if the available stake amount is too low
         """
         stake_amount: float
-        # Ensure wallets are uptodate.
+        # Ensure wallets are up-to-date.
         if update:
             self.update()
         val_tied_up = Trade.total_open_trades_stakes()
         available_amount = self.get_available_stake_amount()
 
-        if edge:
-            stake_amount = edge.stake_amount(
-                pair,
-                self.get_free(self._config['stake_currency']),
-                self.get_total(self._config['stake_currency']),
-                val_tied_up
+        stake_amount = self._config["stake_amount"]
+        if stake_amount == UNLIMITED_STAKE_AMOUNT:
+            stake_amount = self._calculate_unlimited_stake_amount(
+                available_amount, val_tied_up, max_open_trades
             )
-        else:
-            stake_amount = self._config['stake_amount']
-            if stake_amount == UNLIMITED_STAKE_AMOUNT:
-                stake_amount = self._calculate_unlimited_stake_amount(
-                    available_amount, val_tied_up, max_open_trades)
 
         return self._check_available_stake_amount(stake_amount, available_amount)
 
-    def validate_stake_amount(self, pair: str, stake_amount: Optional[float],
-                              min_stake_amount: Optional[float], max_stake_amount: float,
-                              trade_amount: Optional[float]):
-        if not stake_amount:
-            logger.debug(f"Stake amount is {stake_amount}, ignoring possible trade for {pair}.")
+    def validate_stake_amount(
+        self,
+        pair: str,
+        stake_amount: float | None,
+        min_stake_amount: float | None,
+        max_stake_amount: float,
+        trade_amount: float | None,
+    ):
+        if not stake_amount or isinstance(stake_amount, str) or stake_amount <= 0:
+            self._local_log(
+                f"Stake amount is {stake_amount}, ignoring possible trade for {pair}.",
+                level="debug",
+            )
             return 0
 
         max_allowed_stake = min(max_stake_amount, self.get_available_stake_amount())
@@ -341,32 +396,43 @@ class Wallets:
             max_allowed_stake = min(max_allowed_stake, max_stake_amount - trade_amount)
 
         if min_stake_amount is not None and min_stake_amount > max_allowed_stake:
-            if not self._is_backtest:
-                logger.warning("Minimum stake amount > available balance. "
-                               f"{min_stake_amount} > {max_allowed_stake}")
+            self._local_log(
+                "Minimum stake amount > available balance. "
+                f"{min_stake_amount} > {max_allowed_stake}",
+                level="warning",
+            )
             return 0
         if min_stake_amount is not None and stake_amount < min_stake_amount:
-            if not self._is_backtest:
-                logger.info(
-                    f"Stake amount for pair {pair} is too small "
-                    f"({stake_amount} < {min_stake_amount}), adjusting to {min_stake_amount}."
-                )
+            self._local_log(
+                f"Stake amount for pair {pair} is too small "
+                f"({stake_amount} < {min_stake_amount}), adjusting to {min_stake_amount}."
+            )
             if stake_amount * 1.3 < min_stake_amount:
                 # Top-cap stake-amount adjustments to +30%.
-                if not self._is_backtest:
-                    logger.info(
-                        f"Adjusted stake amount for pair {pair} is more than 30% bigger than "
-                        f"the desired stake amount of ({stake_amount:.8f} * 1.3 = "
-                        f"{stake_amount * 1.3:.8f}) < {min_stake_amount}), ignoring trade."
-                    )
+                self._local_log(
+                    f"Adjusted stake amount for pair {pair} is more than 30% bigger than "
+                    f"the desired stake amount of ({stake_amount:.8f} * 1.3 = "
+                    f"{stake_amount * 1.3:.8f}) < {min_stake_amount}), ignoring trade."
+                )
                 return 0
             stake_amount = min_stake_amount
 
         if stake_amount > max_allowed_stake:
-            if not self._is_backtest:
-                logger.info(
-                    f"Stake amount for pair {pair} is too big "
-                    f"({stake_amount} > {max_allowed_stake}), adjusting to {max_allowed_stake}."
-                )
+            self._local_log(
+                f"Stake amount for pair {pair} is too big "
+                f"({stake_amount} > {max_allowed_stake}), adjusting to {max_allowed_stake}."
+            )
             stake_amount = max_allowed_stake
         return stake_amount
+
+    def _local_log(self, msg: str, level: Literal["info", "warning", "debug"] = "info") -> None:
+        """
+        Log a message to the local log.
+        """
+        if not self._is_backtest:
+            if level == "warning":
+                logger.warning(msg)
+            elif level == "debug":
+                logger.debug(msg)
+            else:
+                logger.info(msg)
